package com.sample;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessInstance;
import org.jbpm.process.workitem.wsht.SyncWSHumanTaskHandler;
import org.jbpm.task.AccessType;
import org.jbpm.task.Content;
import org.jbpm.task.TaskService;
import org.jbpm.task.query.TaskSummary;
import org.jbpm.task.service.ContentData;
import org.jbpm.task.service.local.LocalTaskService;
import org.jbpm.test.JbpmJUnitTestCase;
import org.junit.Test;

/**
 * Launching a process.
 */
public class TestingProcess_SUCCESS extends JbpmJUnitTestCase {

	public TestingProcess_SUCCESS() {
		super(true);
		setPersistence(true);
	}

	@Test
	public void testProcess() {
		List<String> groupIds = new ArrayList<String>();
		groupIds.add("user");
		StatefulKnowledgeSession ksession = createKnowledgeSession("TestProcess.bpmn");
		
		/* Commenting this call and substituting it with calling "getTaskServiceAndRegisterMyHumanTaskHandler(ksession)" as shown below as a workaround
		TaskService taskService = getTaskService(ksession); 
		*/
		TaskService taskService = getTaskServiceAndRegisterMyHumanTaskHandler(ksession);
		
		HashMap <String, Object> inputVal = new HashMap <String, Object>();
		
		ProcessInstance processInstance = ksession.startProcess("org.jboss.sample.TestProcess", inputVal);

		assertProcessInstanceActive(processInstance.getId(), ksession);
		assertNodeTriggered(processInstance.getId(), "Start", "First Task");
		
		// let john execute First Task
		List<TaskSummary> list = taskService.getTasksAssignedAsPotentialOwner("john", groupIds, "en-UK");
		TaskSummary task = list.get(0);
		System.out.println("John is executing task " + task.getName());
		System.out.println(task.getStatus());
		taskService.claim(task.getId(), "john",  groupIds);
		taskService.start(task.getId(), "john");
		
		//Print out Content and create ContentData to return results to "First Task"
		Content content = taskService.getContent(task.getId());
		String valKey = "ValkeyOne";
		ByteArrayInputStream bais = new ByteArrayInputStream(content.getContent());
		ContentData contentData = null;
		try {
			ObjectInputStream is = new ObjectInputStream(bais);
			Object obj = null;
			while ((obj = is.readObject()) != null) {
				System.out.println("Printing Value of object: " + obj);
				break;
			}
			if (obj instanceof Map) {
				System.out.println("Found Map Object");
				ByteArrayOutputStream bos = new ByteArrayOutputStream();
				ObjectOutputStream out;
				HashMap<String, Object> contentMap = new HashMap<String, Object>();

				HashMap<String, Object> contentParamMap = (HashMap<String, Object>) obj;
				String value = (String) contentParamMap.get(valKey);
				if (value != null) {
					System.out.println("Printing Value: " + value + " | " + contentParamMap);
					if (value.equals("False")) {
						System.out.println("Setting Value to True");
						contentParamMap.put(valKey, "True");
					} else {
						System.out.println("Setting Value to False");
						contentParamMap.put(valKey, "False");
					}
					try {
						out = new ObjectOutputStream(bos);
						value = (String) contentParamMap.get(valKey);
						System.out.println("Printing Value: " + value);
						contentMap.put("contentParamMap", contentParamMap);
						out.writeObject(contentMap);
						out.close();
						contentData = new ContentData();
						contentData.setContent(bos.toByteArray());
						contentData.setAccessType(AccessType.Inline);
					} catch (IOException e) {
						System.out.println("Error");
						System.err.print(e);
					}
				}

			}
		} catch (Exception e) {
			System.err.print("There was an error reading task input...");
			e.printStackTrace();
		}
		
		if (contentData == null) {
			System.out.println("Content Data is Null");
		}
		taskService.complete(task.getId(), "john", contentData);

		assertNodeTriggered(processInstance.getId(), "Approve");

		ksession.dispose();
	}
	
    private TaskService getTaskServiceAndRegisterMyHumanTaskHandler(
            StatefulKnowledgeSession ksession) {
        LocalTaskService localTaskService = new LocalTaskService(getService());

        SyncWSHumanTaskHandler humanTaskHandler = new SyncWSHumanTaskHandler(
                localTaskService, ksession);
        humanTaskHandler.setLocal(true);
        humanTaskHandler.connect();
        ksession.getWorkItemManager().registerWorkItemHandler("Human Task",
                humanTaskHandler);
        return localTaskService;
    }
	
}